import type { AppearanceMap, InsightModel, NodeQueryResult } from 'ts-common';
import type { GraphQueryResultMetaFromBackend } from 'ts-common/src/model/webSocket/graphResult';
import type { SaveState } from 'ts-common';

export function appearanceCheck(
  result: GraphQueryResultMetaFromBackend,
  insight: InsightModel,
  ss: SaveState,
  queryIndex: number,
  nodeAppearance: AppearanceMap,
) {
  const nameQuery = ss.queryStates.openQueryArray[queryIndex].name;

  if (insight.entityAppearances.nodeLabel == null) {
    throw new Error('No node label chosen, should not continue');
  }

  const queryNodes = result.nodes
    .filter(node => node.label.toLocaleLowerCase() == insight.entityAppearances.nodeLabel.toLocaleLowerCase())
    .map(node => node._id);

  trackEntityAppearances(queryNodes, nodeAppearance, nameQuery || ss.queryStates.openQueryArray[queryIndex].id?.toString() || 'default-id');

  //const sortedAppearance = [...nodeAppearance].sort((a, b) => b.count - a.count);
  //return sortedAppearance;
}

export function sortAndThreshold(appearances: AppearanceMap, insight: InsightModel): AppearanceMap {
  const sortedAppearance = [...appearances].sort((a, b) => b.count - a.count);
  const appearanceTh = sortedAppearance.filter(
    a => a.count > (insight.entityAppearances?.threshold !== undefined ? insight.entityAppearances.threshold : 0),
  );
  return appearanceTh;
}

export function trackEntityAppearances(entities: string[], appearances: AppearanceMap, queryId: string) {
  for (const entity of entities) {
    const existing = appearances.find(entry => entry.nodeID === entity);

    if (existing) {
      existing.count++;
      existing.queries.push(queryId);
    } else {
      appearances.push({ nodeID: entity, count: 1, queries: [queryId] });
    }
  }
}
